#!/bin/bash
VER=1.0
#--[ Intro ]-------------------------------------------------------------#
#                                                                        #
# Tur-AddIp. A script to allow users to add, del and list their own IPs. #
# There are a lot of scripts out there to let users do this, but they    #
# all rely on their own checks to make sure everything is ok (atleast    #
# the ones I've seen).                                                   #
#                                                                        #
# I figured, "Why rely on the script checking everything when glftpd can #
# do it for me?". I mean, all the rules for how the IPs should look are  #
# already in glftpd.conf.                                                #
#                                                                        #
# So, this script uses ncftp to log on as a siteop and add the IP. Then  #
# the rules apply as always. No editing the userfile directly. Just add  #
# it as normal.                                                          #
#                                                                        #
# Since this script holds the username and password for a siteop, it     #
# might be best to install it only if you have trusted shell users on    #
# the box or a jailed enviroment.                                        #
#                                                                        #
#--[ Installation ]------------------------------------------------------#
#                                                                        #
# Copy tur-addip.sh to /glftpd/bin and chmod it to 755.                  #
#                                                                        #
# Copy tur-addip.tcl to your bots config dir and edit it. Change         #
# the mainchan to your irc channel. Users must be in this channel to be  #
# able to /msg botname !addip or they'll be ignored.                     #
# Now add it to the bots config and rehash the bot.                      #
#                                                                        #
# Change the settings.                                                   #
#                                                                        #
#--[ Settings ]----------------------------------------------------------#
#                                                                        #
# ncftp      = Full path to ncftp. Its been tested on                    #
#              NcFTP 3.1.5 (Oct 13, 2002) (ncftp@ncftp.com)              #
#              but other versions should work fine too.                  #
#                                                                        #
# user       = The local user that will do all the adds/dels/lists.      #
#              He MUST have access to the commands user, addip, delip.   #
#                                                                        #
# pass       = The above users password.                                 #
#                                                                        #
# host       = The hostname of the site in the format ftp://hostname     #
#                                                                        #
# port       = The port of the above host.                               #
#                                                                        #
# passchk    = Full path to passchk. Comes with this package but you     #
#              might already have one installed. If you do, make sure    #
#              it gives the output MATCH when running ok. If it gives    #
#              something else, like MATCH! then you can always search    #
#              for MATCH in this script and replace it with MATCH!       #
#              Use 'gcc -o /glftpd/bin/passchk passchk.c' to compile.    #
#              It also comes precompiled on a RH9 box.                   #
#                                                                        #
# passwd     = Full path to glftpds passwd file.                         #
#                                                                        #
# usersdir   = Full path to the dir holding all your users.              #
#                                                                        #
# listonadd  = When adding an IP, do you also want it to show all added  #
#              IPs afterwards? Good reminder for users to clean it up.   #
#                                                                        #
# listondel  = Same as above, but for deleting IPs.                      #
#                                                                        #
# teston     = TRUE/FALSE. Enables the command 'test' which will log in  #
#              and display the information about the defined user above. #
#              Use this to test that all the settings are correct. When  #
#              it looks good, set this to FALSE so users cant run it.    #
#                                                                        #
# log        = File for logging all commands. Set to "" to disable.      #
#                                                                        #
#--[ Info ]--------------------------------------------------------------#
#                                                                        #
# Once you have everything configure, try running it from shell like:    #
# ./tur-addip.sh <username> <password> test                              #
# Make sure teston is set to TRUE first.                                 #
#                                                                        #
# If the above works, su to the user running the bot and try it again.   #
#                                                                        #
# Run it without any args to get a list of available commands (test does #
# not show up in this list).                                             #
#                                                                        #
#--[ Contact ]-----------------------------------------------------------#
#                                                                        #
# Bugs can be reported to turranius on efnet. Usually in #glftpd.        #
# Support on my forums at http://www.grandis.nu/glftpd or                #
# http://grandis.mine.nu (mirror).                                       #
#                                                                        #
# Credits goes to Cruxis for helping me with the tcl channel check.      #
#                                                                        #
#--[ Config area ]-------------------------------------------------------#

ncftp=/usr/bin/ncftp
user=glftpd
pass=glftpd
host=ftp://localhost
port=21

passchk=/glftpd/bin/passchk
passwd=/glftpd/etc/passwd
usersdir=/glftpd/ftp-data/users

listonadd=TRUE
listondel=TRUE

teston=TRUE

log=/glftpd/ftp-data/logs/tur-addip.log

#--[ Script Start ]------------------------------------------------------#

## Procedure for showing help.
proc_help() {
  echo "Tur-Addip $VER Usage:"
  echo "NOTE: Everything is being logged. Abuse leads to deluser!"
  echo "!addip <username> <password> <function>"
  echo "Functions are: "
  echo "list     = List all IP's for the user"
  echo "add <IP> = Adds an IP."
  echo "del <#>  = Dels an IP."
  exit 0
}

proc_log() {
  if [ "$log" ]; then
    echo `date "+%a %b %e %T %Y"`" : $@" >> $log
  fi
}

## Some general checks if everything can be execute and read.
if [ ! -x "$ncftp" ]; then
  echo "Error. Can not execute ncftp. Check path and perms."
  exit 0
elif [ ! -x "$passchk" ]; then
  echo "Error. passchk can not be executed. Check path and perms."
  exit 0
elif [ ! -d "$usersdir" ]; then
  echo "Error. Usersdir does not exist or arent a directory."
  exit 0
elif [ ! -r "$passwd" ]; then
  echo "Error. passwd can not be read. Check path and perms."
  exit 0
fi
if [ "$log" ]; then
  if [ ! -e "$log" ]; then
    echo "Log file; $log does not exist. Create it and set chmod 777 on it."
    exit 0
  elif [ ! -w "$log" ]; then
    echo "I cant write to $log. Set chmod 777 on it."
    exit 0
  fi
fi

## Checking if 3 arguments was used. Show the help otherwise.
if [ -z "$1" -o -z "$2" -o -z "$3" ]; then
  proc_help
fi

## Put all arguments into other arguments.
USERNAME="$1"
PASSWORD="$2"
ACTION="$3"
CURIP="$4"

## Check if the specified user exists.
if [ ! -e "$usersdir/$USERNAME" ]; then
  echo "User $USERNAME does not exist."
  exit 0
fi

## Check if the password matches.
if [ "`$passchk $USERNAME $PASSWORD $passwd`" != "MATCH" ]; then
  echo "Password not accepted for user $USERNAME"
  exit 0
fi

## Procedure for listing IPs
proc_list() {
  echo "Current IPs added to $USERNAME"
  for USERIP in `
  echo "site user $USERNAME" | $ncftp -u $user -p $pass -P $port $host | grep "IP[0-9]\:\ " | tr -d '|' | tr -s ' '
  `; do
    if [ "`echo "$USERIP" | grep "IP[0-9]\:"`" ]; then
      POS="$USERIP"
    else
      echo "$POS $USERIP"
      unset POS
      GOTONE="TRUE"
    fi
  done
  proc_log "$USERNAME checks list of added IPs"
  if [ -z "$GOTONE" ]; then
    echo "No IPs found..."
  fi
}

## Procedure for adding IPs
proc_add() {
  if [ -z "$CURIP" ]; then
    proc_help; exit 0
  elif [ -z "`echo "$CURIP" | grep "\@"`" ]; then
    echo "IP must be in ident@ip format."
    exit 0
  fi

  proc_log "$USERNAME tries to add an IP: $CURIP"

  RESULT=`echo "site addip $USERNAME $CURIP" | ncftp -u $user -p $pass -P $port $host | grep -A1 "Current remote directory is " | grep -v "^Current"`
  RESULT=`echo $RESULT`
  if [ "$RESULT" ]; then
    echo "$RESULT"
  else
    echo "No result. Make sure the user specified as 'user' have addip permissions and that he can log in."
    exit 0
  fi
  if [ "$listonadd" = "TRUE" ]; then
    unset log
    proc_list
  fi
}

## Procedure for deleting IPs
proc_del() {
  if [ -z "$CURIP" ]; then
    proc_help; exit 0
  fi

  proc_log "$USERNAME tries to del an IP: $CURIP"

  RESULT=`echo "site delip $USERNAME $CURIP" | ncftp -u $user -p $pass -P $port $host | grep -A1 "Current remote directory is " | grep -v "^Current "`
  
  RESULT=`echo $RESULT`
  if [ "$RESULT" ]; then
    echo "$RESULT"
  else
    echo "No result. Make sure the user specified as 'user' have delip permissions and that he can log in."
    exit 0
  fi

  if [ "$listondel" = "TRUE" ]; then
    unset log
    proc_list
  fi
}

## Procedure for running the test.
proc_test() {
  if [ "$teston" != "TRUE" ]; then
    proc_help; exit 0
  fi

  proc_log "$USERNAME runs a test."
  echo "Testing that everything works as it should."
  echo "It should log in and show the userstats for $user"
  echo "Make sure that $user has access to '-usersothers', '-addip' & '-delip'"
  echo 'Running: echo "site user '$USERNAME'" | ncftp -u '$user' -p '$pass' -P '$port' '$host''

  until [ -n "$go" ]; do
    echo -n "Ready to go? [Y]es [N]o: "
    read go
    case $go in
      [Nn])
        exit 0
        continue
        ;;
      [Yy])
        echo " "
        go=n
        ;;
      *)
       unset go
       continue
       ;;
    esac
  done
  unset go

  echo ""
  echo "site user $USERNAME" | ncftp -u $user -p $pass -P $port $host  
  echo ""
  echo "If you see the info for $user above the login is good. You must make sure he has"
  echo "permissions to list other users, add ips and del ips"
}

## Main menu.
case $ACTION in
  [lL][iI][sS][tT]) proc_list; exit 0;;
  [aA][dD][dD])     proc_add;  exit 0;;
  [dD][eE][lL])     proc_del;  exit 0;;
  [tT][eE][sS][tT]) proc_test; exit 0;;
  *)                proc_help; exit 0;;
esac

echo "How did I get here? bug."
exit 0
